import argparse
import math
import sys
import time
from dataclasses import dataclass
from typing import List, Type

import matplotlib.pyplot as plt
import numpy as np

from .plot import (AnimatedFigure, CPUPlotter, GPUPlotter, IOPlotter,
                   MemoryPlotter, NetworkPlotter)

if sys.platform == 'linux':
    DEFAULT_PLOTTER_CLASSES = (CPUPlotter, GPUPlotter, MemoryPlotter, IOPlotter, NetworkPlotter)
else:
    DEFAULT_PLOTTER_CLASSES = (CPUPlotter, MemoryPlotter, IOPlotter, NetworkPlotter)

@dataclass
class ToppyArgs:
    update_interval: float = 1.0
    display_seconds: float = 60.0
    plotter_classes: list = DEFAULT_PLOTTER_CLASSES

    @classmethod
    def parse_args(cls, args=None):
        parser = argparse.ArgumentParser()
        parser.add_argument('-i', '--update-interval', type=float,
                            help='update interval ( >= 0.01 )')
        parser.add_argument('-s', '--display_seconds', type=float,
                            help='number of seconds to show ( >= 1 )')
        args = parser.parse_args(args)
        if not cls._validate_args(args):
            parser.print_usage()
            parser.exit()
        present_args = {k: v for k, v in args.__dict__.items() if v is not None}
        return cls(**present_args)

    @staticmethod
    def _validate_args(args):
        if args.update_interval is not None and args.update_interval < 0.01: return False
        if args.display_seconds is not None and args.display_seconds < 1: return False
        return True



class Toppy:
    def __init__(self, args: ToppyArgs = None):
        self.args = args or ToppyArgs()

    def run(self, block=True):
        plotters = [plotter_cls() for plotter_cls in self.args.plotter_classes]
        fig = plt.figure('Toppy')
        animated_figure = AnimatedFigure(
            plotters,
            self.args.update_interval,
            self.args.display_seconds,
            fig
        )
        animated_figure.run(block=block)



import psutil


class MemoryStat:
    def __init__(self):
        self.mem = None
        self.swap = None


    def setup(self):
        self._update()

    def update(self):
        self._update()

    def _update(self):
        self.mem = psutil.virtual_memory()
        self.swap = psutil.swap_memory()



import time

import matplotlib.animation as animation
import matplotlib.pyplot as plt
import numpy as np


class AnimatedAxes:
    def setup(self, axes, x):
        raise NotImplementedError(f'Please implement {self.__class__.__name__}.setup(axes, x)')

    def update(self):
        raise NotImplementedError(f'Please implement {self.__class__.__name__}.update()')


class AnimatedFigure:
    def __init__(self, animated_axes, update_interval, display_seconds, fig=None):
        self.animated_axes = animated_axes
        self.update_interval = update_interval
        self.display_seconds = display_seconds
        self.fig = fig or plt.figure()
        self.setup_actors = None

    def run(self, block=True):
        self.anim = animation.FuncAnimation(
            self.fig,
            self.update,
            init_func=self.setup,
            interval=self.update_interval * 1000,
            save_count=1,
            cache_frame_data=False,
            blit=True
        )
        plt.show(block=block)

    def setup(self):
        self.setup_actors = self.setup_actors or list(self._setup_animated_axes())
        return self.setup_actors

    def _setup_animated_axes(self):
        x = np.arange(0, -self.display_seconds, -self.update_interval)[::-1]
        axes = tolist(self.fig.subplots(len(self.animated_axes), 1))
        for animated_axes, axes in zip(self.animated_axes, axes):
            yield from tolist(animated_axes.setup(axes, x))

    def update(self, frame):
        actors = list(self._update_animated_axes())
        return actors

    def _update_animated_axes(self):
        for animated_axes in self.animated_axes:
            yield from tolist(animated_axes.update())


def tolist(obj):
    if type(obj) in (list, np.ndarray):
        return obj
    return [obj]



import numpy as np

from ..system_stat import CPUStat
from . import common
from .animated import AnimatedAxes


class CPUPlotter(AnimatedAxes):
    def __init__(self, cpu=None):
        self.cpu = cpu or CPUStat()

    def setup(self, axes, x):
        self.cpu.setup()

        self.y = common.none_array(x.size)

        axes.set_title('CPU')
        axes.set_ylabel('% CPU')
        axes.set_xlim(x.min(), x.max())
        axes.set_ylim(0, 100)
        axes.tick_params('x', bottom=False, labelbottom=False)
        axes.grid(True, axis='y')

        self.line = axes.plot(x, self.y)[0]
        return self.line

    def update(self):
        self.cpu.update()

        common.additem_cyclic_inplace(self.y, self.cpu.utilization)

        self.line.set_ydata(self.y)
        return self.line



from .animated import AnimatedAxes, AnimatedFigure
from .cpu_plotter import CPUPlotter
from .gpu_plotter import GPUPlotter
from .io_plotter import IOPlotter
from .memory_plotter import MemoryPlotter
from .network_plotter import NetworkPlotter



# -*- coding: utf-8 -*-

from setuptools import find_packages, setup

with open('README.md') as f:
    readme = f.read()

setup(
    name='toppy',
    version='0.1.2',
    description='Graphical system rsources monitor in python',
    long_description=readme,
    long_description_content_type="text/markdown",
    author='Roi Gabay',
    author_email='roigby@gmail.com',
    url='https://github.com/gabay/toppy',
    license='MIT',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    packages=find_packages(exclude=('test',)),
    install_requires=[
        'matplotlib>=3.3.4,<4',
        'psutil>=5.8.0,<6',
        'gpustat>=0.6.0,<1'
    ]
)



import time
from collections import namedtuple
import psutil

netstat = namedtuple('netstat', 'rx tx')

def make_netstat(old_stats, new_stats, timediff):
    rx = (new_stats.bytes_recv - old_stats.bytes_recv) / timediff
    tx = (new_stats.bytes_sent - old_stats.bytes_sent) / timediff
    return netstat(rx, tx)

def active_devices(stats):
    for key in stats:
        if key == 'lo':
            continue
        if stats[key].bytes_recv == 0 and stats[key].bytes_sent == 0:
            continue
        yield key

class NetworkStat:
    def __init__(self):
        self._stats = None
        self._time = None

        self.keys = None
        self.stats = None

    def setup(self):
        self._update()
        self.keys = list(active_devices(self._stats))

    def update(self):
        old_stats, old_time = self._stats, self._time
        self._update()
        new_stats, new_time = self._stats, self._time
        timediff_s = new_time - old_time

        keys = set(old_stats.keys()) & set(new_stats.keys())
        self.stats = {
            key: make_netstat(old_stats[key], new_stats[key], timediff_s) for key in keys
        }

    def _update(self):
        self._stats = psutil.net_io_counters(pernic=True)
        self._time = time.time()
